COMMENT
ihw.mod
based on
isin.mod
but omits isin.mod's phase parameter phi.

Delivers half-wave rectified sinusoidal current.

The current generator is off until one of these two things happens:
t = del
or an external event arrives at time t = tx.
Once on, the generator stays on for dur ms, then turns back off.
External or endogenous activating events that arrive
while the generator is on are ignored.

Current is
i = amp*hw(f*(t - tstart))
where
amp is peak ampltude (may be + or -)
f is frequency in radians/s
hw(theta) = sin(theta) if sin(theta)>=0
            0 if sin(theta)<0
tstart is either del or the time at which an external event arrived.

The current starts at t==start and ends at t==tstart+dur.

9/4/2013 NTC
ENDCOMMENT

NEURON {
  POINT_PROCESS Ihw
  RANGE del, dur, f, amp, i
  ELECTRODE_CURRENT i
}

UNITS {
  (nA) = (nanoamp)
  PI = (pi) (1)
  (Radian) = (1)
}

PARAMETER {
  del (ms)           : time at which current starts
  dur (ms) <0, 1e9>  : duration
  f (Radian/s) <0, 1e9> : frequency in radians/s
  amp (nA)
}

ASSIGNED {
  i (nA)
  on (1)
  tstart (ms)
}

INITIAL {
  i = 0
  on = 0
  net_send(del, 1)
}

FUNCTION hw(tx (ms)) {
  hw = sin(f*tx/(1000))
  if (hw<0) {
    hw = 0
  }
}

BREAKPOINT {
  i = on*amp*hw(t-tstart)
}

NET_RECEIVE (w) {
  if (on == 0) { : it's off
    if ((flag == 0) || (flag == 1)) { : an "activating event"
      tstart = t
      on = 1 : turn it on
      net_send(dur, 2) : prepare to turn it off
    }
  } else { : already on, ignore "activating events"
    if (flag == 2) {
      on = 0
    }
  }
}

